"""
.. module:: teamScraper
   :synopsis: A module for scraping team schedule and roster information.

.. moduleauthor:: Erin McNulty
"""

import requests
from bs4 import BeautifulSoup
import re

BASE_URL = "https://play.usaultimate.org"


def getTeamInfo(**kwargs):
    '''getTeamInfo() returns all information about the first 20 teams matching the query

    Args:
        ``schoolName`` (string): The school that the team is affiliated with (ex. Columbia).

        ``teamName`` (string): The name of the team (ex. Pleiades).

        ``genderDivision`` (string): The gender division that the team competes in.
        Must be one of: Women, Men, Mixed, Boys, Girls.

        ``state`` (string): Postal abbreviation of the state the team is in.

        ``competitionLevel`` (string): The competition level that the team plays at.
        Must be one of: Club, College, High School, or Middle School.

        ``competitionDivision`` (string): The competition division that the team plays in.
        Must be one of: Classic, Developmental, Division 1, or Division 3.

        ``teamDesignation`` (string): If the team is a B or a C team. Must be B or C.

        ``teamURI`` (string): If querying based on team URI, will simply return the results of the team with
        this URI andignore other parameters.

    Returns:
        results:
            ::

                {
                    res: OK, NOTFOUND
                    teams: [
                        {
                            schoolName,
                            teamName,
                            competitionLevel,
                            genderDivision,
                            location,
                            coaches,
                            website,
                            facebook,
                            twitter,
                        },
                        ...
                    ]
                }

    '''
    teams = queryTeam(kwargs)

    if len(teams) == 0:
        return {"res": "NOTFOUND"}

    with requests.Session() as req:
        res = {"res": "OK", "teams": []}

        for _, endpoint in teams.items():
            r = req.get(BASE_URL + endpoint)
            soup = BeautifulSoup(r.content, 'html.parser')

            team = fillInBasicInfo(soup)

            coaches = soup.find(id="CT_Main_0_ucTeamDetails_dlHeadCoach")
            if coaches is not None:
                team["coaches"] = re.sub(r"[^a-zA-Z0-9 - , ]", "", coaches.find("dd").getText())

            website = soup.find(id="CT_Main_0_ucTeamDetails_dlWebsite")
            if website is not None:
                team["website"] = website.find("a").getText()

            facebook = soup.find(id="CT_Main_0_ucTeamDetails_dlFacebook")
            if facebook is not None:
                team["facebook"] = facebook.find("a").getText()

            twitter = soup.find(id="CT_Main_0_ucTeamDetails_dlTwitter")
            if twitter is not None:
                team["twitter"] = twitter.find("a").getText()

            res["teams"].append(team)

        return res


def getTeamSchedule(**kwargs):
    '''getTeamSchedule() returns the season schedule and record of the first 20 teams matching the query

    Args:
        ``schoolName`` (string): The school that the team is affiliated with (ex. Columbia).

        ``teamName`` (string): The name of the team (ex. Pleiades).

        ``genderDivision`` (string): The gender division that the team competes in.
        Must be one of: Women, Men, Mixed, Boys, Girls.

        ``state`` (string): Postal abbreviation of the state the team is in.

        ``competitionLevel`` (string): The competition level that the team plays at.
        Must be one of: Club, College, High School, or Middle School.

        ``competitionDivision`` (string): The competition division that the team plays in.
        Must be one of: Classic, Developmental, Division 1, or Division 3.

        ``teamDesignation`` (string): If the team is a B or a C team. Must be B or C.

        ``teamURI`` (string): If querying based on team URI, will simply return the results of the team with
        this URI and ignore other parameters.

    Returns:
        ::

            {
                res: OK, NOTFOUND
                teams: [
                    {
                        schoolName,
                        teamName,
                        competitionLevel,
                        genderDivision,
                        wins,
                        losses,
                        tournaments: {
                            name: {
                                games: [
                                    {
                                        date,
                                        score,
                                        opponentCollege,
                                        opponentTeamPage
                                    },
                                    ...
                                ]
                            },
                            ...
                        },
                    },
                    ...
                ]
            }'''
    teams = queryTeam(kwargs)

    if len(teams) == 0:
        return {"res": "NOTFOUND"}

    with requests.Session() as req:
        res = {"res": "OK", "teams": []}

        for _, endpoint in teams.items():
            r = req.get(BASE_URL + endpoint)
            soup = BeautifulSoup(r.content, 'html.parser')

            team = fillInBasicInfo(soup)
            team["wins"] = 0
            team["losses"] = 0
            team["tournaments"] = {}

            scheduleTable = soup.find(id="CT_Right_0_gvEventScheduleScores")

            if scheduleTable is None:
                res["teams"].append(team)
                continue

            scheduleTableRows = scheduleTable.findAll("tr")
            currentTournament = ""

            for row in scheduleTableRows:
                cells = row.findAll("td")

                if len(cells) == 1:
                    currentTournament = cells[0].find("a").getText()
                    team["tournaments"][currentTournament] = {"games": []}
                else:
                    date = cells[0].find("span").getText()
                    score = cells[1].find("a").getText()
                    oppCollege = cells[2].find("a").getText()
                    oppHref = cells[2].find("a").get("href")

                    game = {"date": date, "score": score, "opponentCollege": oppCollege, "opponentHref": oppHref}

                    if not cells[1].get("class") is None:
                        if cells[1].get("class")[0] == "win":
                            team["wins"] += 1
                        elif cells[1].get("class")[0] == "loss":
                            team["losses"] += 1

                    team["tournaments"][currentTournament]["games"].append(game)

            res["teams"].append(team)

        return res


def getTeamRoster(**kwargs):
    '''getTeamRoster() returns the roster of the first 20 teams matching the query

    Args:
        ``schoolName`` (string): The school that the team is affiliated with (ex. Columbia).

        ``teamName`` (string): The name of the team (ex. Pleiades).

        ``genderDivision`` (string): The gender division that the team competes in.
        Must be one of: Women, Men, Mixed, Boys, Girls.

        ``state`` (string): Postal abbreviation of the state the team is in.

        ``competitionLevel`` (string): The competition level that the team plays at.
        Must be one of: Club, College, High School, or Middle School.

        ``competitionDivision`` (string): The competition division that the team plays in.
        Must be one of: Classic, Developmental, Division 1, or Division 3.

        ``teamDesignation`` (string): If the team is a B or a C team. Must be B or C.

        ``teamURI`` (string): If querying based on team URI, will simply return the results of the team with
        this URI and ignore other parameters.

    Returns:
        results:
            ::

                {
                    res: OK, NOTFOUND
                    teams: [
                        {
                            schoolName,
                            teamName,
                            competitionLevel,
                            genderDivision,
                            roster: [
                                {
                                    no,
                                    name,
                                    pronouns,
                                    position,
                                    year,
                                    height,
                                },
                                ...
                            ]
                        },
                        ...
                    ]
                }
    '''

    teams = queryTeam(kwargs)

    if len(teams) == 0:
        return {"res": "NOTFOUND"}

    with requests.Session() as req:
        res = {"res": "OK", "teams": []}

        for _, endpoint in teams.items():
            r = req.get(BASE_URL + endpoint)
            soup = BeautifulSoup(r.content, 'html.parser')

            team = fillInBasicInfo(soup)

            team["roster"] = []

            rosterTable = soup.find(id="CT_Main_0_ucTeamDetails_gvList")

            rosterRows = rosterTable.findAll("tr")

            for row in rosterRows[1:]:
                cells = row.findAll("td")
                player = {}
                player["no"] = cells[0].getText()
                player["name"] = cells[1].getText()
                player["pronouns"] = cells[2].getText().strip()
                player["position"] = cells[3].getText().strip()
                player["year"] = cells[4].getText().strip()
                player["height"] = cells[5].getText().strip()

                team["roster"].append(player)

            res["teams"].append(team)

        return res


def fillInBasicInfo(soup):
    team = {}
    schoolTeam = soup.find(class_="profile_info").find("h4").getText()
    schoolTeamList = schoolTeam.split(" (")

    team["schoolName"] = schoolTeamList[0].strip()
    team["teamName"] = schoolTeamList[1].strip()[:-1]
    team["competitionLevel"] = soup.find(id="CT_Main_0_ucTeamDetails_dlCompetitionLevel").find("dd").getText()
    team["genderDivision"] = soup.find(id="CT_Main_0_ucTeamDetails_dlGenderDivision").find("dd").getText()
    team["location"] = soup.find(id="CT_Main_0_ucTeamDetails_dlCity").getText().strip()

    return team


def queryTeam(args):
    if "teamURI" in args:
        return {"singleTeam": args["teamURI"]}

    with requests.Session() as req:
        endpoint = "/teams/events/rankings/"
        teamDict = {}
        r = req.get(BASE_URL + endpoint)
        soup = BeautifulSoup(r.content, 'html.parser')

        data = setArgs(args)
        data['__VIEWSTATE'] = soup.find("input", id="__VIEWSTATE").get("value")
        data['__VIEWSTATEGENERATOR'] = soup.find("input", id="__VIEWSTATEGENERATOR").get("value")
        data['__EVENTVALIDATION'] = soup.find("input", id="__EVENTVALIDATION").get("value")
        r = req.post(BASE_URL + endpoint, data=data)
        soup = BeautifulSoup(r.content, 'html.parser')

        links = soup.findAll(id=re.compile("lnkTeam"))

        for link in links:
            teamDict[link.getText()] = link.get("href")

        return teamDict


def setArgs(args):
    designation_mappings = {
        "Gender": {"Boys": 20, "Girls": 19, "Men": 17, "Mixed": 1, "Women": 2},
        "Division": {"Classic": 10, "Developmental": 4, "Division 1": 1, "Division 3": 3},
        "Designation": {"B": 1, "C": 2},
    }

    checkArgs(designation_mappings, args)

    data = {
        "__EVENTTARGET": "CT_Main_0$gvList$ctl23$ctl00$ctl00",
        "CT_Main_0$F_Status": "Published",
        "CT_Main_0$F_SchoolName": args["schoolName"] if "schoolName" in args else "",
        "CT_Main_0$F_TeamName": args["teamName"] if "teamName" in args else "",
        "CT_Main_0$F_GenderDivisionId": designation_mappings["Gender"][args["genderDivision"]]
        if "genderDivision" in args
        else "",
        "CT_Main_0$F_StateId": args["state"] if "state" in args else "",
        "CT_Main_0$F_CompetitionLevelId": args["competitionLevel"] if "competitionLevel" in args else "",
        "CT_Main_0$F_CompetitionDivisionId": designation_mappings["Division"][args["competitionDivision"]]
        if "competitionDivision" in args
        else "",
        "CT_Main_0$F_Designation": designation_mappings["Designation"][args["teamDesignation"]]
        if "teamDesignation" in args
        else "",
    }

    return data


def checkArgs(designation_mappings, args):
    if "genderDivision" in args:
        if args["genderDivision"] not in designation_mappings["Gender"]:
            raise ValueError("Gender Division must be one of: Boys, Girls, Men, Mixed, Women")

    if "competitionDivision" in args:
        if args["competitionDivision"] not in designation_mappings["Division"]:
            raise ValueError("Competition Division must be one of: Classic, Developmental, Division 1, Division 3")

    if "teamDesignation" in args:
        if args["teamDesignation"] not in designation_mappings["Designation"]:
            raise ValueError("Competition Designation must be B or C")

    if "competitionLevel" in args:
        if args["competitionLevel"] not in ["Club", "College", "High School", "Middle School"]:
            raise ValueError("Competition Level must be one of: Club, College, High School, Middle School")



"""
.. module:: tournamentScraper
   :synopsis: A module for scraping tournament results

.. moduleauthor:: Erin McNulty
"""

import requests
from bs4 import BeautifulSoup
import pandas as pd
import re

BASE_URL = "https://play.usaultimate.org"


def getTournamentPoolPlayResults(competitionLevel, genderDivision, **kwargs):
    '''getTournamentPoolPlayResults() returns the pool play and consolation results of a tournament.

        If the query returns multiple tournaments, returns the result of the first tournament that
        appears on the usau site which the query is issued.

    Args:
        ``competitionLevel`` (string) (required): The competition level of the tournament.
        Must be one of: Club, College, or High School.

        ``genderDivision`` (string) (required): The gender division to view tournament results for.
        Must be one of: Women, Men, Mixed, Boys, Girls.

        ``eventName`` (string): The name of the tournament.

        ``state`` (string): Postal abbreviation of the state the tournament was in.

        ``season`` (int): The season that the tournament occured in.
        Must be after 2006

    Returns:
        results:
            ::

                {
                    res: OK, NOTFOUND
                    tournamentName: {
                        pools: {
                            poolName: [
                                {
                                    name,
                                    wins,
                                    losses
                                },
                                ...
                            ],
                            ...
                        },
                        rounds: {
                            roundName: {
                                sectionName: [
                                    {
                                        date,
                                        time,
                                        field,
                                        team1Name,
                                        team2Name,
                                        score,
                                        status
                                    },
                                    ...
                                ]
                            }
                        }
                    }
                }


    '''
    tournamentName, tournamentLink = queryTournament(competitionLevel, genderDivision, kwargs)

    if tournamentName == '':
        return {"res": "NOTFOUND"}

    with requests.Session() as req:
        response = {"res": "OK"}
        response[tournamentName] = {}
        response[tournamentName]["pools"] = {}
        response[tournamentName]["rounds"] = {}

        r = req.get(tournamentLink + "/schedule/" + genderDivision + "/" + competitionLevel + genderDivision)
        soup = BeautifulSoup(r.content, 'html.parser')

        roundNames = soup.find_all(id=re.compile("CT_Main_0_rptTabs_ctl.._liTab"))
        index2RoundName = []

        for listItem in roundNames:
            roundName = listItem.find("a").getText()
            response[tournamentName]["rounds"][roundName] = {}
            index2RoundName.append(roundName)

        roundResultsDiv = soup.find("div", {"class": "slides"})
        roundResults = roundResultsDiv.find_all("section", {"class": "slide"})

        i = 0
        for roundResult in roundResults:
            if roundResult.get("id") == "poolSlide":
                # ----------------- Add in pool results to pool play -------------------
                pools = roundResult.find_all("div", {"class": "pool"})

                for pool in pools:
                    poolName = pool.find("h3").getText()
                    response[tournamentName]["pools"][poolName] = []

                    poolTable = pool.find("table")
                    poolResults = pd.read_html(str(poolTable), na_values=None)

                    for poolResult in poolResults:
                        response[tournamentName]["pools"][poolName] = poolResult.to_dict("records")

                # ----------------- Add in non-bracket results -------------------
                sections = roundResult.find_all("table", {"class": "scores_table"})

                for section in sections:
                    sectionName = section.find("th").getText()
                    sectionResults = pd.read_html(str(section), header=0, na_values=None)

                    for sectionResult in sectionResults:
                        sectionResult.columns = sectionResult.iloc[0]
                        sectionResult = sectionResult[1:]
                        sectionResult = sectionResult.drop("Options", axis=1)
                        response[tournamentName]["rounds"][index2RoundName[i]][sectionName] = sectionResult.to_dict(
                            "records"
                        )

            if response[tournamentName]["rounds"][index2RoundName[i]] == {}:
                del response[tournamentName]["rounds"][index2RoundName[i]]

            i += 1

        return response


def getTournamentBracketResults(competitionLevel, genderDivision, **kwargs):
    '''getTournamentBracketResults() returns the bracket results of a tournament.

    If the query returns multiple tournaments, returns the result of the first tournament that
    appears on the usau site which the query is issued.

    Args:
        ``competitionLevel`` (string) (required): The competition level of the tournament.
        Must be one of: Club, College, or High School.

        ``genderDivision`` (string) (required): The gender division to view tournament results for.
        Must be one of: Women, Men, Mixed, Boys, Girls.

        ``eventName`` (string): The name of the tournament.

        ``state`` (string): Postal abbreviation of the state the tournament was in.

        ``season`` (int): The season that the tournament occured in.
        Must be after 2006

    Returns:
        results:
            ::

                {
                    res: OK, NOTFOUND
                    tournamentName: {
                        bracketName: {
                            roundName: [
                                {
                                    date,
                                    time,
                                    field,
                                    winner,
                                    loser,
                                    score,
                                    status
                                },
                                ...
                            ]
                        }
                    }
                }
    '''
    tournamentName, tournamentLink = queryTournament(competitionLevel, genderDivision, kwargs)

    if tournamentName == '':
        return {"res": "NOTFOUND"}

    with requests.Session() as req:
        response = {"res": "OK"}
        response[tournamentName] = {}

        r = req.get(tournamentLink + "/schedule/" + genderDivision + "/" + competitionLevel + genderDivision)
        soup = BeautifulSoup(r.content, 'html.parser')

        roundResults = soup.find("div", {"class": "slides"}).find_all("section", {"class": "slide"})

        for roundResult in roundResults:
            if roundResult.get("id") == "bracketSlide":
                # -------------- Add in bracket results --------------------
                sections = roundResult.find_all("section", {"class": "section page"})

                for section in sections:
                    sectionName = section.find("h3").getText().strip()
                    response[tournamentName][sectionName] = {}

                    rounds = section.find_all("div", {"class": "bracket_col"})

                    for r in rounds:
                        roundName = r.find("h4").getText()
                        response[tournamentName][sectionName][roundName] = []

                        games = r.find_all("div", {"class": "bracket_game"})

                        for game in games:
                            status = game.find("span", {"class": "game-status"}).getText()

                            if status == "Final":
                                date, time, am_pm = game.find("span", {"class": "date"}).getText().split(" ")
                                time = time + am_pm
                                field = game.find("p", {"class": "location"}).getText()
                                winnerDiv = game.find("div", {"class": "winner"})
                                loserDiv = game.find("div", {"class": "loser"})
                                winner = winnerDiv.find("span", {"class": "isName"}).find("a").getText()
                                loser = loserDiv.find("span", {"class": "isName"}).find("a").getText()
                                scoreW = winnerDiv.find("span", {"class": "isScore"}).getText().strip()
                                scoreL = loserDiv.find("span", {"class": "isScore"}).getText().strip()
                                score = scoreW + " - " + scoreL

                                gameDict = {
                                    "date": date,
                                    "time": time,
                                    "field": field,
                                    "winner": winner,
                                    "loser": loser,
                                    "score": score,
                                    "status": status,
                                }

                                response[tournamentName][sectionName][roundName].append(gameDict)

        return response


def getTournamentWinner(competitionLevel, genderDivision, **kwargs):
    '''getTournamentWinner() returns the first and second place results of a tournament.

    If the query returns multiple tournaments, returns the result of the first tournament that
    appears on the usau site which the query is issued.

    Args:
        ``competitionLevel`` (string) (required): The competition level of the tournament.
        Must be one of: Club, College, or High School.

        ``genderDivision`` (string) (required): The gender division to view tournament results for.
        Must be one of: Women, Men, Mixed, Boys, Girls.

        ``eventName`` (string): The name of the tournament.

        ``state`` (string): Postal abbreviation of the state the tournament was in.

        ``season`` (int): The season that the tournament occured in.
        Must be after 2006

    Returns:
        results:
            ::

                {
                    res: OK, NOTFOUND
                    tournamentName: {
                        firstPlace,
                        secondPlace
                    }
                }
    '''
    bracketPlayResults = getTournamentBracketResults(competitionLevel, genderDivision, **kwargs)

    if bracketPlayResults["res"] == "NOTFOUND":
        return bracketPlayResults

    tournamentName = list(bracketPlayResults)[1]
    firstBracket = list(bracketPlayResults[tournamentName])[0]
    firstGame = list(bracketPlayResults[tournamentName][firstBracket])[0]

    tournamentResDict = {
        "firstPlace": bracketPlayResults[tournamentName][firstBracket][firstGame][0]["winner"],
        "secondPlace": bracketPlayResults[tournamentName][firstBracket][firstGame][0]["loser"],
    }

    res = {"res": "OK"}
    res[tournamentName] = tournamentResDict

    return res


def queryTournament(competitionLevel, genderDivision, args):
    if "tournamentURI" in args:
        return {"singleTournament": args["tournamentURI"]}

    with requests.Session() as req:
        endpoint = "/events/tournament/"

        r = req.get(BASE_URL + endpoint, params=setArgs(competitionLevel, genderDivision, args))
        soup = BeautifulSoup(r.content, 'html.parser')

        tournaments = soup.find("div", {"class": "search-results"}).findChildren("tr")

        if len(tournaments) > 1:
            tournamentName = tournaments[1].find("a").getText()
            tournamentLink = tournaments[1].find("a").get("href")

            return tournamentName, tournamentLink

        return '', ''


def seasonToId(season):
    if season > 2018:
        return season - 2005
    elif season > 2010:
        return season - 2010
    else:
        return 2019 - season


def setArgs(competitionLevel, genderDivision, args):
    designation_mappings = {
        'College - Mixed': 4,
        'College - Women': 28,
        'College - Men': 27,
        'High School - Mixed': 3,
        'High School - Girls': 30,
        'High School - Boys': 29,
        'Club - Women': 22,
        'Club - Mixed': 7,
        'Club - Men': 21,
    }

    checkArgs(competitionLevel, genderDivision, args)

    competitionLevelId = designation_mappings[competitionLevel + " - " + genderDivision]

    queryParams = {
        'ViewAll': 'true',
        'IsLeagueType': 'false',
        'IsClinic': 'false',
        'FilterByCategory': 'AT',
        'CompetitionLevelId': competitionLevelId,
        'SeasonId': seasonToId(args["season"]) if "season" in args else "",
        'EventName': args["eventName"] if "eventName" in args else "",
        'DateFrom': args["dateFrom"] if "dateFrom" in args else "",
        'DateTo': args["dateTo"] if "dateTo" in args else "",
    }

    return queryParams


def checkArgs(competitionLevel, genderDivision, args):
    if genderDivision not in ['Boys', 'Girls', 'Men', 'Mixed', 'Women']:
        raise ValueError("Gender Division must be one of: Boys, Girls, Men, Mixed, Women")

    if competitionLevel not in ["Club", "College", "High School"]:
        raise ValueError("Competition Level must be one of: Club, College, High School")

    if "season" in args:
        if args["season"] < 2006:
            raise ValueError("Season must be after 2006")



from .teamScraper import getTeamInfo, getTeamSchedule, getTeamRoster, queryTeam, setArgs, fillInBasicInfo
from .tournamentScraper import (
    queryTournament,
    getTournamentWinner,
    getTournamentPoolPlayResults,
    getTournamentBracketResults,
)
from .rankingScraper import getClubRankings, getCollegeRankings, queryRankings

__all__ = [
    'getTeamInfo',
    'getTeamSchedule',
    'getTeamRoster',
    'queryTeam',
    'setArgs',
    'fillInBasicInfo',
    'queryTournament',
    'getTournamentWinner',
    'getTournamentPoolPlayResults',
    'getTournamentBracketResults',
    'getCollegeRankings',
    'getClubRankings',
    'queryRankings',
    '__version__',
]
__version__ = "0.4.0"



"""
.. module:: rankingScraper
   :synopsis: A module for scraping team rankings.

.. moduleauthor:: Cherie Liu
"""

import requests
from bs4 import BeautifulSoup
import pandas as pd

BASE_URL = "https://play.usaultimate.org"


def getCollegeRankings(**kwargs):
    '''getCollegeRankings() returns top 20 teams for inputted gender at the college competition level.

    Args:
        ``genderDivision`` (string): Women, Men, or Mixed (for Club only)

    Returns:
        results:
            ::

                {
                    res: OK, NOTFOUND
                    teams: [
                        {
                            rank,
                            team,
                            powerRating,
                            competitionLevel,
                            genderDivison,
                            competitionDivision,
                            collegeRegion,
                            collegeConference,
                            wins,
                            losses
                        },
                        ...
                    ]
                }
    '''
    kwargs["competitionLevel"] = "College"
    teams = queryRankings(kwargs)
    if len(teams) == 0:
        return {"res": "NOTFOUND"}
    else:
        return {"res": "OK", "teams": teams}


def getClubRankings(**kwargs):
    '''getClubRankings() returns top 20 teams for inputted gender at the club competition level.

    Args:
        ``genderDivision`` (string): Women, Men, or Mixed (for Club only)

    Returns:
        results:
            ::

                {
                    res: OK, NOTFOUND
                    teams: [
                        {
                            rank,
                            team,
                            powerRating,
                            genderDivison,
                            competitionDivision,
                            city,
                            state,
                            clubRegion,
                            clubSection,
                            wins,
                            losses
                        },
                        ...
                    ]
                }
    '''
    kwargs["competitionLevel"] = "Club"
    teams = queryRankings(kwargs)
    if len(teams) == 0:
        return {"res": "NOTFOUND"}
    else:
        return {"res": "OK", "teams": teams}


def queryRankings(args):
    with requests.Session() as req:
        endpoint = "/teams/events/team_rankings/?RankSet=" + args["competitionLevel"] + "-" + args["genderDivision"]
        r = req.get(BASE_URL + endpoint)
        soup = BeautifulSoup(r.content, 'html.parser')

        r = req.post(BASE_URL + endpoint)
        soup = BeautifulSoup(r.content, 'html.parser')

        table = soup.find('table', id="CT_Main_0_gvList")
        dfs = pd.read_html(str(table))[0].head(20)
        teamsDict = dfs.to_dict('records')

        return teamsDict



import requests
from bs4 import BeautifulSoup

from usau_scraper import (
    queryTeam,
    getTeamRoster,
    getTeamInfo,
    getTeamSchedule,
    setArgs,
    fillInBasicInfo,
    getTournamentBracketResults,
    getTournamentPoolPlayResults,
    getTournamentWinner,
    getCollegeRankings,
    getClubRankings,
)

# ------------------------------- UNIT TESTS -------------------------------


# ------------------ queryTeam Tests ------------------
def test_query_team_multiple_teams():
    team_results = queryTeam({"schoolName": "Columbia", "competitionLevel": "College", "genderDivision": "Women"})

    expected_res_keys = ["British Columbia (Thunderbirds)", "Columbia (Curbside)", "Columbia-B (Baewatch)"]

    assert list(team_results.keys()) == expected_res_keys


def test_query_team_no_teams():
    team_results = queryTeam({"schoolName": "wiugwiugbiwug"})

    assert team_results == {}


def test_query_team_more_than_twenty():
    team_results = queryTeam({"competitionLevel": "College"})

    assert len(team_results) == 20


def test_query_team_uri_passed_in():
    teams = queryTeam({"teamURI": "TEST_URI"})

    assert len(teams) == 1
    assert teams["singleTeam"] == "TEST_URI"


# ------------------ setArgs Tests ------------------
def test_set_args():
    args = {
        "schoolName": "Columbia",
        "teamName": "Baewatch",
        "genderDivision": "Women",
        "state": "NY",
        "competitionLevel": "College",
        "competitionDivision": "Division 1",
        "teamDesignation": "B",
    }

    expectedData = {
        "__EVENTTARGET": "CT_Main_0$gvList$ctl23$ctl00$ctl00",
        "CT_Main_0$F_Status": "Published",
        "CT_Main_0$F_SchoolName": "Columbia",
        "CT_Main_0$F_TeamName": "Baewatch",
        "CT_Main_0$F_GenderDivisionId": 2,
        "CT_Main_0$F_StateId": "NY",
        "CT_Main_0$F_CompetitionLevelId": "College",
        "CT_Main_0$F_CompetitionDivisionId": 1,
        "CT_Main_0$F_Designation": 1,
    }

    data = setArgs(args)

    assert data == expectedData


# ------------------ fillInBasicInfo Tests ------------------
def test_basic_info():
    with requests.Session() as req:
        r = req.get(
            "https://play.usaultimate.org/teams/events/Eventteam/?TeamId=xPLDyRaZDSiR0OTzDBsYfzn8PJeIKdalFgjWpKUXiLI%3d"
        )
        soup = BeautifulSoup(r.content, 'html.parser')

        info = fillInBasicInfo(soup)

        expectedInfo = {
            "schoolName": "Columbia",
            "teamName": "Curbside",
            "competitionLevel": "College",
            "genderDivision": "Women",
            "location": "New York , New York",
        }

        assert info == expectedInfo


# ------------------------------- INTEGRATION TESTS -------------------------------


# ------------------ getTeamInfo Tests ------------------
def test_get_team_info():
    teams = getTeamInfo(schoolName="North Carolina", teamName="Pleiades")

    expectedTeamInfo = [
        {
            "schoolName": "North Carolina",
            "teamName": "Pleiades",
            "competitionLevel": "College",
            "genderDivision": "Women",
            "location": "Chapel Hill, North Carolina",
            "coaches": "Jessica Jones Head CoachMary Rippe Elisabeth Parker",
            "website": "http://www.uncpleiades.com",
            "facebook": "https://www.facebook.com/unc.pleiades/",
            "twitter": "https://twitter.com/UNC_Pleiades/",
        }
    ]

    assert teams["res"] == "OK"
    assert teams["teams"] == expectedTeamInfo


def test_get_team_info_not_found():
    teams = getTeamInfo(schoolName="wgiubwiugbiwugbwiubwgub")

    assert teams["res"] == "NOTFOUND"


# ------------------ getTeamSchedule Tests ------------------
def test_get_team_schedule():
    teams = getTeamSchedule(schoolName="Columbia", teamName="Curbside")

    expectedTournamentResults = {
        "games": [
            {
                "date": "March 04",
                "score": "7 - 9",
                "opponentCollege": "Cornell",
                "opponentHref": "/teams/events/Eventteam/?TeamId=swZShzB%2bFK4tsqtqa%2f511SwW9nXxGho7MjxCYzUxwlM%3d",
            },
            {
                "date": "March 04",
                "score": "9 - 6",
                "opponentCollege": "Rutgers",
                "opponentHref": "/teams/events/Eventteam/?TeamId=Cx6jfxJr%2bTJsYl3Z3urhSz4tpQxIcvCualgJhm1k%2b4w%3d",
            },
            {
                "date": "March 04",
                "score": "8 - 6",
                "opponentCollege": "Bates",
                "opponentHref": "/teams/events/Eventteam/?TeamId=qmVAo7N%2ftRZfIFCu881VvxKKXav4UKHaB842B%2bV%2fOD8%3d",
            },
            {
                "date": "March 05",
                "score": "6 - 8",
                "opponentCollege": "Williams",
                "opponentHref": "/teams/events/Eventteam/?TeamId=t1%2bUZ9UKbj6kJdnTioIuZWSnPEnaeyxXCvaufWX7QOQ%3d",
            },
            {
                "date": "March 05",
                "score": "8 - 5",
                "opponentCollege": "Harvard",
                "opponentHref": "/teams/events/Eventteam/?TeamId=Qes6Me2QohbI%2faNOUJQwedpQQwIdMXQLxWGTGRzTie8%3d",
            },
        ]
    }

    assert teams["res"] == "OK"
    assert teams["teams"][0]["tournaments"]["No Sleep Till Brooklyn 2023"] == expectedTournamentResults


# ------------------ getTeamRoster Tests ------------------
def test_get_team_roster():
    teams = getTeamRoster(schoolName="Columbia", teamName="Curbside")

    expectedPlayer = {
        "no": "7",
        "name": "Aya Lahlou",
        "pronouns": "S",
        "position": "",
        "year": "College (GR)",
        "height": "5'5\"",
    }

    assert teams["res"] == "OK"
    assert expectedPlayer in teams["teams"][0]["roster"]


# ------------------ getTournamentPoolPlayResults Tests ------------------
def test_get_tournament_pool_play():
    results = getTournamentPoolPlayResults("College", "Women", eventName="No Sleep Till Brooklyn", season=2023)

    expectedPoolAResults = {
        "Team": "Wellesley (1)",
        "W - L": "3 - 0",
    }

    assert results["res"] == "OK"
    del results["No Sleep Till Brooklyn 2023"]["pools"]["Pool A"][0]["Tie"]  # bc NaN != NaN
    assert results["No Sleep Till Brooklyn 2023"]["pools"]["Pool A"][0] == expectedPoolAResults


# ------------------ getTournamentBracketResults Tests ------------------
def test_get_tournament_bracket():
    results = getTournamentBracketResults("College", "Women", eventName="No Sleep Till Brooklyn", season=2023)

    expectedFinalsResults = {
        "date": "3/5/2023",
        "time": "1:00PM",
        "field": "8b",
        "winner": "Wellesley (1)",
        "loser": "Cornell (11)",
        "score": "10 - 4",
        "status": "Final",
    }

    assert results["res"] == "OK"
    assert results["No Sleep Till Brooklyn 2023"]["Championship Bracket"]["Finals"][0] == expectedFinalsResults


# ------------------ getTournamentWinner Tests ------------------
def test_get_tournament_winner():
    results = getTournamentWinner("College", "Women", eventName="No Sleep Till Brooklyn", season=2023)

    expectedResults = {"firstPlace": "Wellesley (1)", "secondPlace": "Cornell (11)"}

    assert results["res"] == "OK"
    assert results["No Sleep Till Brooklyn 2023"] == expectedResults


# ------------------ getCollegeRankings Tests ------------------
def test_get_college_rankings():
    teams = getCollegeRankings(genderDivision="Women")

    expectedTeam = {
        "Rank": "1",
        "Team": "North Carolina",
        "Power Rating": "2824",
        "Competition Level": "College",
        "Gender Division": "Women",
        "Competition Division": "Division I",
        "College Region": "Atlantic Coast",
        "College Conference": "Carolina DI",
        "Wins": "16",
        "Losses": "0",
    }

    assert teams["res"] == "OK"
    assert expectedTeam == teams["teams"][0]


# ------------------ getClubRankings Tests ------------------
def test_get_club_rankings():
    teams = getClubRankings(genderDivision="Men")

    expectedTeam = {
        "Rank": "1",
        "Team": "Truck Stop",
        "Power Rating": "2134",
        "Gender Division": "Men",
        "Competition Division": "Pro",
        "City": "Washington",
        "State": "DC",
        "Club Region": "Mid-Atlantic",
        "Club Section": "Capital",
        "Wins": "30",
        "Losses": "4",
    }

    assert teams["res"] == "OK"
    assert expectedTeam == teams["teams"][0]
